@@ -33,35 +33,49 @@ BlobLogWriter::BlobLogWriter(std::unique_ptr<WritableFileWriter>&& dest,
 
 BlobLogWriter::~BlobLogWriter() = default;
 
-Status BlobLogWriter::Sync() {
+Status BlobLogWriter::Sync(const WriteOptions& write_options) {
   TEST_SYNC_POINT("BlobLogWriter::Sync");
 
   StopWatch sync_sw(clock_, statistics_, BLOB_DB_BLOB_FILE_SYNC_MICROS);
-  Status s = dest_->Sync(use_fsync_);
-  RecordTick(statistics_, BLOB_DB_BLOB_FILE_SYNCED);
+  IOOptions opts;
+  Status s = WritableFileWriter::PrepareIOOptions(write_options, opts);
+  if (s.ok()) {
+    s = dest_->Sync(opts, use_fsync_);
+  }
+  if (s.ok()) {
+    RecordTick(statistics_, BLOB_DB_BLOB_FILE_SYNCED);
+  }
   return s;
 }
 
-Status BlobLogWriter::WriteHeader(BlobLogHeader& header) {
+Status BlobLogWriter::WriteHeader(const WriteOptions& write_options,
+                                  BlobLogHeader& header) {
   assert(block_offset_ == 0);
   assert(last_elem_type_ == kEtNone);
   std::string str;
   header.EncodeTo(&str);
 
-  Status s = dest_->Append(Slice(str));
+  IOOptions opts;
+  Status s = WritableFileWriter::PrepareIOOptions(write_options, opts);
+  if (s.ok()) {
+    s = dest_->Append(opts, Slice(str));
+  }
   if (s.ok()) {
     block_offset_ += str.size();
     if (do_flush_) {
-      s = dest_->Flush();
+      s = dest_->Flush(opts);
     }
   }
   last_elem_type_ = kEtFileHdr;
-  RecordTick(statistics_, BLOB_DB_BLOB_FILE_BYTES_WRITTEN,
-             BlobLogHeader::kSize);
+  if (s.ok()) {
+    RecordTick(statistics_, BLOB_DB_BLOB_FILE_BYTES_WRITTEN,
+               BlobLogHeader::kSize);
+  }
   return s;
 }
 
-Status BlobLogWriter::AppendFooter(BlobLogFooter& footer,
+Status BlobLogWriter::AppendFooter(const WriteOptions& write_options,
+                                   BlobLogFooter& footer,
                                    std::string* checksum_method,
                                    std::string* checksum_value) {
   assert(block_offset_ != 0);
@@ -75,14 +89,17 @@ Status BlobLogWriter::AppendFooter(BlobLogFooter& footer,
     s.PermitUncheckedError();
     return Status::IOError("Seen Error. Skip closing.");
   } else {
-    s = dest_->Append(Slice(str));
+    IOOptions opts;
+    s = WritableFileWriter::PrepareIOOptions(write_options, opts);
+    if (s.ok()) {
+      s = dest_->Append(opts, Slice(str));
+    }
     if (s.ok()) {
       block_offset_ += str.size();
-
-      s = Sync();
+      s = Sync(write_options);
 
       if (s.ok()) {
-        s = dest_->Close();
+        s = dest_->Close(opts);
 
         if (s.ok()) {
           assert(!!checksum_method == !!checksum_value);
@@ -111,12 +128,15 @@ Status BlobLogWriter::AppendFooter(BlobLogFooter& footer,
   }
 
   last_elem_type_ = kEtFileFooter;
-  RecordTick(statistics_, BLOB_DB_BLOB_FILE_BYTES_WRITTEN,
-             BlobLogFooter::kSize);
+  if (s.ok()) {
+    RecordTick(statistics_, BLOB_DB_BLOB_FILE_BYTES_WRITTEN,
+               BlobLogFooter::kSize);
+  }
   return s;
 }
 
-Status BlobLogWriter::AddRecord(const Slice& key, const Slice& val,
+Status BlobLogWriter::AddRecord(const WriteOptions& write_options,
+                                const Slice& key, const Slice& val,
                                 uint64_t expiration, uint64_t* key_offset,
                                 uint64_t* blob_offset) {
   assert(block_offset_ != 0);
@@ -125,11 +145,13 @@ Status BlobLogWriter::AddRecord(const Slice& key, const Slice& val,
   std::string buf;
   ConstructBlobHeader(&buf, key, val, expiration);
 
-  Status s = EmitPhysicalRecord(buf, key, val, key_offset, blob_offset);
+  Status s =
+      EmitPhysicalRecord(write_options, buf, key, val, key_offset, blob_offset);
   return s;
 }
 
-Status BlobLogWriter::AddRecord(const Slice& key, const Slice& val,
+Status BlobLogWriter::AddRecord(const WriteOptions& write_options,
+                                const Slice& key, const Slice& val,
                                 uint64_t* key_offset, uint64_t* blob_offset) {
   assert(block_offset_ != 0);
   assert(last_elem_type_ == kEtFileHdr || last_elem_type_ == kEtRecord);
@@ -137,7 +159,8 @@ Status BlobLogWriter::AddRecord(const Slice& key, const Slice& val,
   std::string buf;
   ConstructBlobHeader(&buf, key, val, 0);
 
-  Status s = EmitPhysicalRecord(buf, key, val, key_offset, blob_offset);
+  Status s =
+      EmitPhysicalRecord(write_options, buf, key, val, key_offset, blob_offset);
   return s;
 }
 
@@ -150,28 +173,34 @@ void BlobLogWriter::ConstructBlobHeader(std::string* buf, const Slice& key,
   record.EncodeHeaderTo(buf);
 }
 
-Status BlobLogWriter::EmitPhysicalRecord(const std::string& headerbuf,
+Status BlobLogWriter::EmitPhysicalRecord(const WriteOptions& write_options,
+                                         const std::string& headerbuf,
                                          const Slice& key, const Slice& val,
                                          uint64_t* key_offset,
                                          uint64_t* blob_offset) {
-  StopWatch write_sw(clock_, statistics_, BLOB_DB_BLOB_FILE_WRITE_MICROS);
-  Status s = dest_->Append(Slice(headerbuf));
+  IOOptions opts;
+  Status s = WritableFileWriter::PrepareIOOptions(write_options, opts);
   if (s.ok()) {
-    s = dest_->Append(key);
+    s = dest_->Append(opts, Slice(headerbuf));
   }
   if (s.ok()) {
-    s = dest_->Append(val);
+    s = dest_->Append(opts, key);
+  }
+  if (s.ok()) {
+    s = dest_->Append(opts, val);
   }
   if (do_flush_ && s.ok()) {
-    s = dest_->Flush();
+    s = dest_->Flush(opts);
   }
 
   *key_offset = block_offset_ + BlobLogRecord::kHeaderSize;
   *blob_offset = *key_offset + key.size();
   block_offset_ = *blob_offset + val.size();
   last_elem_type_ = kEtRecord;
-  RecordTick(statistics_, BLOB_DB_BLOB_FILE_BYTES_WRITTEN,
-             BlobLogRecord::kHeaderSize + key.size() + val.size());
+  if (s.ok()) {
+    RecordTick(statistics_, BLOB_DB_BLOB_FILE_BYTES_WRITTEN,
+               BlobLogRecord::kHeaderSize + key.size() + val.size());
+  }
   return s;
 }
 
