@@ -24,7 +24,10 @@ Reader::Reporter::~Reporter() = default;
 
 Reader::Reader(std::shared_ptr<Logger> info_log,
                std::unique_ptr<SequentialFileReader>&& _file,
-               Reporter* reporter, bool checksum, uint64_t log_num)
+               Reporter* reporter, bool checksum, uint64_t log_num,
+               bool track_and_verify_wals, bool stop_replay_for_corruption,
+               uint64_t min_wal_number_to_keep,
+               const PredecessorWALInfo& observed_predecessor_wal_info)
     : info_log_(info_log),
       file_(std::move(_file)),
       reporter_(reporter),
@@ -37,13 +40,17 @@ Reader::Reader(std::shared_ptr<Logger> info_log,
       last_record_offset_(0),
       end_of_buffer_offset_(0),
       log_number_(log_num),
+      track_and_verify_wals_(track_and_verify_wals),
+      stop_replay_for_corruption_(stop_replay_for_corruption),
+      min_wal_number_to_keep_(min_wal_number_to_keep),
+      observed_predecessor_wal_info_(observed_predecessor_wal_info),
       recycled_(false),
       first_record_read_(false),
       compression_type_(kNoCompression),
       compression_type_record_read_(false),
       uncompress_(nullptr),
       hash_state_(nullptr),
-      uncompress_hash_state_(nullptr){}
+      uncompress_hash_state_(nullptr) {}
 
 Reader::~Reader() {
   delete[] backing_store_;
@@ -65,6 +72,9 @@ Reader::~Reader() {
 //
 // TODO krad: Evaluate if we need to move to a more strict mode where we
 // restrict the inconsistency to only the last log
+// TODO (hx235): move `wal_recovery_mode` to be a member data like other
+// information (e.g, `stop_replay_for_corruption`) to decide whether to
+// check for and surface corruption in `ReadRecord()`
 bool Reader::ReadRecord(Slice* record, std::string* scratch,
                         WALRecoveryMode wal_recovery_mode,
                         uint64_t* record_checksum) {
@@ -85,10 +95,10 @@ bool Reader::ReadRecord(Slice* record, std::string* scratch,
   uint64_t prospective_record_offset = 0;
 
   Slice fragment;
-  while (true) {
+  for (;;) {
     uint64_t physical_record_offset = end_of_buffer_offset_ - buffer_.size();
     size_t drop_size = 0;
-    const unsigned int record_type =
+    const uint8_t record_type =
         ReadPhysicalRecord(&fragment, &drop_size, record_checksum);
     switch (record_type) {
       case kFullType:
@@ -130,7 +140,7 @@ bool Reader::ReadRecord(Slice* record, std::string* scratch,
         prospective_record_offset = physical_record_offset;
         scratch->assign(fragment.data(), fragment.size());
         in_fragmented_record = true;
-        break;
+        break;  // switch
 
       case kMiddleType:
       case kRecyclableMiddleType:
@@ -143,7 +153,7 @@ bool Reader::ReadRecord(Slice* record, std::string* scratch,
           }
           scratch->append(fragment.data(), fragment.size());
         }
-        break;
+        break;  // switch
 
       case kLastType:
       case kRecyclableLastType:
@@ -161,7 +171,7 @@ bool Reader::ReadRecord(Slice* record, std::string* scratch,
           first_record_read_ = true;
           return true;
         }
-        break;
+        break;  // switch
 
       case kSetCompressionType: {
         if (compression_type_record_read_) {
@@ -183,7 +193,24 @@ bool Reader::ReadRecord(Slice* record, std::string* scratch,
         } else {
           InitCompression(compression_record);
         }
-        break;
+        break;  // switch
+      }
+      case kPredecessorWALInfoType:
+      case kRecyclePredecessorWALInfoType: {
+        prospective_record_offset = physical_record_offset;
+        scratch->clear();
+        last_record_offset_ = prospective_record_offset;
+
+        PredecessorWALInfo recorded_predecessor_wal_info;
+        Status s = recorded_predecessor_wal_info.DecodeFrom(&fragment);
+        if (!s.ok()) {
+          ReportCorruption(fragment.size(),
+                           "could not decode PredecessorWALInfoType record");
+        } else {
+          MaybeVerifyPredecessorWALInfo(wal_recovery_mode, fragment,
+                                        recorded_predecessor_wal_info);
+        }
+        break;  // switch
       }
       case kUserDefinedTimestampSizeType:
       case kRecyclableUserDefinedTimestampSizeType: {
@@ -208,7 +235,7 @@ bool Reader::ReadRecord(Slice* record, std::string* scratch,
             ReportCorruption(fragment.size(), s.getState());
           }
         }
-        break;
+        break;  // switch
       }
 
       case kBadHeader:
@@ -277,7 +304,7 @@ bool Reader::ReadRecord(Slice* record, std::string* scratch,
           in_fragmented_record = false;
           scratch->clear();
         }
-        break;
+        break;  // switch
 
       case kBadRecordLen:
         if (eof_) {
@@ -310,21 +337,78 @@ bool Reader::ReadRecord(Slice* record, std::string* scratch,
           in_fragmented_record = false;
           scratch->clear();
         }
-        break;
+        break;  // switch
 
       default: {
-        std::string reason =
-            "unknown record type " + std::to_string(record_type);
-        ReportCorruption(
-            (fragment.size() + (in_fragmented_record ? scratch->size() : 0)),
-            reason.c_str());
+        if ((record_type & kRecordTypeSafeIgnoreMask) == 0) {
+          std::string reason =
+              "unknown record type " + std::to_string(record_type);
+          ReportCorruption(
+              (fragment.size() + (in_fragmented_record ? scratch->size() : 0)),
+              reason.c_str());
+        }
         in_fragmented_record = false;
         scratch->clear();
-        break;
+        break;  // switch
       }
     }
   }
-  return false;
+  // unreachable
+}
+
+void Reader::MaybeVerifyPredecessorWALInfo(
+    WALRecoveryMode wal_recovery_mode, Slice fragment,
+    const PredecessorWALInfo& recorded_predecessor_wal_info) {
+  if (!track_and_verify_wals_ ||
+      wal_recovery_mode == WALRecoveryMode::kSkipAnyCorruptedRecords ||
+      stop_replay_for_corruption_) {
+    return;
+  }
+  assert(recorded_predecessor_wal_info.IsInitialized());
+  uint64_t recorded_predecessor_log_number =
+      recorded_predecessor_wal_info.GetLogNumber();
+
+  // This is the first WAL recovered thus with no predecessor WAL info has been
+  // initialized
+  if (!observed_predecessor_wal_info_.IsInitialized()) {
+    if (recorded_predecessor_log_number >= min_wal_number_to_keep_) {
+      std::string reason = "Missing WAL of log number " +
+                           std::to_string(recorded_predecessor_log_number);
+      ReportCorruption(fragment.size(), reason.c_str(),
+                       recorded_predecessor_log_number);
+    }
+  } else {
+    if (observed_predecessor_wal_info_.GetLogNumber() !=
+        recorded_predecessor_log_number) {
+      std::string reason = "Missing WAL of log number " +
+                           std::to_string(recorded_predecessor_log_number);
+      ReportCorruption(fragment.size(), reason.c_str(),
+                       recorded_predecessor_log_number);
+    } else if (observed_predecessor_wal_info_.GetLastSeqnoRecorded() !=
+               recorded_predecessor_wal_info.GetLastSeqnoRecorded()) {
+      std::string reason =
+          "Mismatched last sequence number recorded in the WAL of log number " +
+          std::to_string(recorded_predecessor_log_number) + ". Recorded " +
+          std::to_string(recorded_predecessor_wal_info.GetLastSeqnoRecorded()) +
+          ". Observed " +
+          std::to_string(
+              observed_predecessor_wal_info_.GetLastSeqnoRecorded()) +
+          ". (Last sequence number equal to 0 indicates no WAL records)";
+      ReportCorruption(fragment.size(), reason.c_str(),
+                       recorded_predecessor_log_number);
+    } else if (observed_predecessor_wal_info_.GetSizeBytes() !=
+               recorded_predecessor_wal_info.GetSizeBytes()) {
+      std::string reason =
+          "Mismatched size of the WAL of log number " +
+          std::to_string(recorded_predecessor_log_number) + ". Recorded " +
+          std::to_string(recorded_predecessor_wal_info.GetSizeBytes()) +
+          " bytes. Observed " +
+          std::to_string(observed_predecessor_wal_info_.GetSizeBytes()) +
+          " bytes.";
+      ReportCorruption(fragment.size(), reason.c_str(),
+                       recorded_predecessor_log_number);
+    }
+  }
 }
 
 uint64_t Reader::LastRecordOffset() { return last_record_offset_; }
@@ -403,13 +487,15 @@ void Reader::UnmarkEOFInternal() {
   }
 }
 
-void Reader::ReportCorruption(size_t bytes, const char* reason) {
-  ReportDrop(bytes, Status::Corruption(reason));
+void Reader::ReportCorruption(size_t bytes, const char* reason,
+                              uint64_t log_number) {
+  ReportDrop(bytes, Status::Corruption(reason), log_number);
 }
 
-void Reader::ReportDrop(size_t bytes, const Status& reason) {
+void Reader::ReportDrop(size_t bytes, const Status& reason,
+                        uint64_t log_number) {
   if (reporter_ != nullptr) {
-    reporter_->Corruption(bytes, reason);
+    reporter_->Corruption(bytes, reason, log_number);
   }
 }
 
@@ -419,7 +505,7 @@ void Reader::ReportOldLogRecord(size_t bytes) {
   }
 }
 
-bool Reader::ReadMore(size_t* drop_size, int* error) {
+bool Reader::ReadMore(size_t* drop_size, uint8_t* error) {
   if (!eof_ && !read_error_) {
     // Last read was a full read, so this is a trailer to skip
     buffer_.clear();
@@ -460,15 +546,15 @@ bool Reader::ReadMore(size_t* drop_size, int* error) {
   }
 }
 
-unsigned int Reader::ReadPhysicalRecord(Slice* result, size_t* drop_size,
-                                        uint64_t* fragment_checksum) {
+uint8_t Reader::ReadPhysicalRecord(Slice* result, size_t* drop_size,
+                                   uint64_t* fragment_checksum) {
   while (true) {
     // We need at least the minimum header size
     if (buffer_.size() < static_cast<size_t>(kHeaderSize)) {
       // the default value of r is meaningless because ReadMore will overwrite
       // it if it returns false; in case it returns true, the return value will
       // not be used anyway
-      int r = kEof;
+      uint8_t r = kEof;
       if (!ReadMore(drop_size, &r)) {
         return r;
       }
@@ -479,12 +565,13 @@ unsigned int Reader::ReadPhysicalRecord(Slice* result, size_t* drop_size,
     const char* header = buffer_.data();
     const uint32_t a = static_cast<uint32_t>(header[4]) & 0xff;
     const uint32_t b = static_cast<uint32_t>(header[5]) & 0xff;
-    const unsigned int type = header[6];
+    const uint8_t type = static_cast<uint8_t>(header[6]);
     const uint32_t length = a | (b << 8);
     int header_size = kHeaderSize;
     const bool is_recyclable_type =
         ((type >= kRecyclableFullType && type <= kRecyclableLastType) ||
-         type == kRecyclableUserDefinedTimestampSizeType);
+         type == kRecyclableUserDefinedTimestampSizeType ||
+         type == kRecyclePredecessorWALInfoType);
     if (is_recyclable_type) {
       header_size = kRecyclableHeaderSize;
       if (first_record_read_ && !recycled_) {
@@ -494,7 +581,7 @@ unsigned int Reader::ReadPhysicalRecord(Slice* result, size_t* drop_size,
       recycled_ = true;
       // We need enough for the larger header
       if (buffer_.size() < static_cast<size_t>(kRecyclableHeaderSize)) {
-        int r = kEof;
+        uint8_t r = kEof;
         if (!ReadMore(drop_size, &r)) {
           return r;
         }
@@ -549,6 +636,8 @@ unsigned int Reader::ReadPhysicalRecord(Slice* result, size_t* drop_size,
     buffer_.remove_prefix(header_size + length);
 
     if (!uncompress_ || type == kSetCompressionType ||
+        type == kPredecessorWALInfoType ||
+        type == kRecyclePredecessorWALInfoType ||
         type == kUserDefinedTimestampSizeType ||
         type == kRecyclableUserDefinedTimestampSizeType) {
       *result = Slice(header + header_size, length);
@@ -638,7 +727,9 @@ Status Reader::UpdateRecordedTimestampSize(
 }
 
 bool FragmentBufferedReader::ReadRecord(Slice* record, std::string* scratch,
-                                        WALRecoveryMode /*unused*/,
+                                        WALRecoveryMode wal_recovery_mode
+
+                                        ,
                                         uint64_t* /* checksum */) {
   assert(record != nullptr);
   assert(scratch != nullptr);
@@ -651,7 +742,7 @@ bool FragmentBufferedReader::ReadRecord(Slice* record, std::string* scratch,
   uint64_t prospective_record_offset = 0;
   uint64_t physical_record_offset = end_of_buffer_offset_ - buffer_.size();
   size_t drop_size = 0;
-  unsigned int fragment_type_or_err = 0;  // Initialize to make compiler happy
+  uint8_t fragment_type_or_err = 0;  // Initialize to make compiler happy
   Slice fragment;
   while (TryReadFragment(&fragment, &drop_size, &fragment_type_or_err)) {
     switch (fragment_type_or_err) {
@@ -728,7 +819,24 @@ bool FragmentBufferedReader::ReadRecord(Slice* record, std::string* scratch,
         }
         break;
       }
+      case kPredecessorWALInfoType:
+      case kRecyclePredecessorWALInfoType: {
+        fragments_.clear();
+        prospective_record_offset = physical_record_offset;
+        last_record_offset_ = prospective_record_offset;
+        in_fragmented_record_ = false;
 
+        PredecessorWALInfo recorded_predecessor_wal_info;
+        Status s = recorded_predecessor_wal_info.DecodeFrom(&fragment);
+        if (!s.ok()) {
+          ReportCorruption(fragment.size(),
+                           "could not decode PredecessorWALInfoType record");
+        } else {
+          MaybeVerifyPredecessorWALInfo(wal_recovery_mode, fragment,
+                                        recorded_predecessor_wal_info);
+        }
+        break;
+      }
       case kUserDefinedTimestampSizeType:
       case kRecyclableUserDefinedTimestampSizeType: {
         if (in_fragmented_record_ && !scratch->empty()) {
@@ -781,11 +889,13 @@ bool FragmentBufferedReader::ReadRecord(Slice* record, std::string* scratch,
         break;
 
       default: {
-        std::string reason =
-            "unknown record type " + std::to_string(fragment_type_or_err);
-        ReportCorruption(
-            fragment.size() + (in_fragmented_record_ ? fragments_.size() : 0),
-            reason.c_str());
+        if ((fragment_type_or_err & kRecordTypeSafeIgnoreMask) == 0) {
+          std::string reason =
+              "unknown record type " + std::to_string(fragment_type_or_err);
+          ReportCorruption(
+              fragment.size() + (in_fragmented_record_ ? fragments_.size() : 0),
+              reason.c_str());
+        }
         in_fragmented_record_ = false;
         fragments_.clear();
         break;
@@ -803,7 +913,7 @@ void FragmentBufferedReader::UnmarkEOF() {
   UnmarkEOFInternal();
 }
 
-bool FragmentBufferedReader::TryReadMore(size_t* drop_size, int* error) {
+bool FragmentBufferedReader::TryReadMore(size_t* drop_size, uint8_t* error) {
   if (!eof_ && !read_error_) {
     // Last read was a full read, so this is a trailer to skip
     buffer_.clear();
@@ -844,15 +954,15 @@ bool FragmentBufferedReader::TryReadMore(size_t* drop_size, int* error) {
 }
 
 // return true if the caller should process the fragment_type_or_err.
-bool FragmentBufferedReader::TryReadFragment(
-    Slice* fragment, size_t* drop_size, unsigned int* fragment_type_or_err) {
+bool FragmentBufferedReader::TryReadFragment(Slice* fragment, size_t* drop_size,
+                                             uint8_t* fragment_type_or_err) {
   assert(fragment != nullptr);
   assert(drop_size != nullptr);
   assert(fragment_type_or_err != nullptr);
 
   while (buffer_.size() < static_cast<size_t>(kHeaderSize)) {
     size_t old_size = buffer_.size();
-    int error = kEof;
+    uint8_t error = kEof;
     if (!TryReadMore(drop_size, &error)) {
       *fragment_type_or_err = error;
       return false;
@@ -863,11 +973,12 @@ bool FragmentBufferedReader::TryReadFragment(
   const char* header = buffer_.data();
   const uint32_t a = static_cast<uint32_t>(header[4]) & 0xff;
   const uint32_t b = static_cast<uint32_t>(header[5]) & 0xff;
-  const unsigned int type = header[6];
+  const uint8_t type = static_cast<uint8_t>(header[6]);
   const uint32_t length = a | (b << 8);
   int header_size = kHeaderSize;
   if ((type >= kRecyclableFullType && type <= kRecyclableLastType) ||
-      type == kRecyclableUserDefinedTimestampSizeType) {
+      type == kRecyclableUserDefinedTimestampSizeType ||
+      type == kRecyclePredecessorWALInfoType) {
     if (first_record_read_ && !recycled_) {
       // A recycled log should have started with a recycled record
       *fragment_type_or_err = kBadRecord;
@@ -877,7 +988,7 @@ bool FragmentBufferedReader::TryReadFragment(
     header_size = kRecyclableHeaderSize;
     while (buffer_.size() < static_cast<size_t>(kRecyclableHeaderSize)) {
       size_t old_size = buffer_.size();
-      int error = kEof;
+      uint8_t error = kEof;
       if (!TryReadMore(drop_size, &error)) {
         *fragment_type_or_err = error;
         return false;
@@ -894,7 +1005,7 @@ bool FragmentBufferedReader::TryReadFragment(
 
   while (header_size + length > buffer_.size()) {
     size_t old_size = buffer_.size();
-    int error = kEof;
+    uint8_t error = kEof;
     if (!TryReadMore(drop_size, &error)) {
       *fragment_type_or_err = error;
       return false;
@@ -923,6 +1034,8 @@ bool FragmentBufferedReader::TryReadFragment(
   buffer_.remove_prefix(header_size + length);
 
   if (!uncompress_ || type == kSetCompressionType ||
+      type == kPredecessorWALInfoType ||
+      type == kRecyclePredecessorWALInfoType ||
       type == kUserDefinedTimestampSizeType ||
       type == kRecyclableUserDefinedTimestampSizeType) {
     *fragment = Slice(header + header_size, length);
