@@ -13,6 +13,7 @@
 #include <unordered_map>
 #include <vector>
 
+#include "db/dbformat.h"
 #include "db/log_format.h"
 #include "rocksdb/compression_type.h"
 #include "rocksdb/env.h"
@@ -76,18 +77,24 @@ class Writer {
   // Create a writer that will append data to "*dest".
   // "*dest" must be initially empty.
   // "*dest" must remain live while this Writer is in use.
+  // TODO(hx235): seperate WAL related parameters from general `Reader`
+  // parameters
   explicit Writer(std::unique_ptr<WritableFileWriter>&& dest,
                   uint64_t log_number, bool recycle_log_files,
                   bool manual_flush = false,
-                  CompressionType compressionType = kNoCompression);
+                  CompressionType compressionType = kNoCompression,
+                  bool track_and_verify_wals = false);
   // No copying allowed
   Writer(const Writer&) = delete;
   void operator=(const Writer&) = delete;
 
   ~Writer();
 
-  IOStatus AddRecord(const WriteOptions& write_options, const Slice& slice);
+  IOStatus AddRecord(const WriteOptions& write_options, const Slice& slice,
+                     const SequenceNumber& seqno = 0);
   IOStatus AddCompressionTypeRecord(const WriteOptions& write_options);
+  IOStatus MaybeAddPredecessorWALInfo(const WriteOptions& write_options,
+                                      const PredecessorWALInfo& info);
 
   // If there are column families in `cf_to_ts_sz` not included in
   // `recorded_cf_to_ts_sz_` and its user-defined timestamp size is non-zero,
@@ -116,6 +123,8 @@ class Writer {
 
   size_t TEST_block_offset() const { return block_offset_; }
 
+  SequenceNumber GetLastSeqnoRecorded() const { return last_seqno_recorded_; };
+
  private:
   std::unique_ptr<WritableFileWriter> dest_;
   size_t block_offset_;  // Current offset in block
@@ -131,6 +140,11 @@ class Writer {
   IOStatus EmitPhysicalRecord(const WriteOptions& write_options,
                               RecordType type, const char* ptr, size_t length);
 
+  IOStatus MaybeHandleSeenFileWriterError();
+
+  IOStatus MaybeSwitchToNewBlock(const WriteOptions& write_options,
+                                 const std::string& content_to_write);
+
   // If true, it does not flush after each write. Instead it relies on the upper
   // layer to manually does the flush by calling ::WriteBuffer()
   bool manual_flush_;
@@ -145,6 +159,11 @@ class Writer {
   // Since the user-defined timestamp size cannot be changed while the DB is
   // running, existing entry in this map cannot be updated.
   UnorderedMap<uint32_t, size_t> recorded_cf_to_ts_sz_;
+
+  // See `Options::track_and_verify_wals`
+  bool track_and_verify_wals_;
+
+  SequenceNumber last_seqno_recorded_;
 };
 
 }  // namespace log
