@@ -38,7 +38,12 @@ public class ConfigOptions extends RocksObject {
     return this;
   }
 
-  @Override protected final native void disposeInternal(final long handle);
+  @Override
+  protected final void disposeInternal(final long handle) {
+    disposeInternalJni(handle);
+  }
+
+  private static native void disposeInternalJni(final long handle);
 
   private static long newConfigOptionsInstance() {
     RocksDB.loadLibrary();
