@@ -938,76 +938,6 @@ public interface DBOptionsInterface<T extends DBOptionsInterface<T>> {
    */
   long dbWriteBufferSize();
 
-  /**
-   * Specify the file access pattern once a compaction is started.
-   * It will be applied to all input files of a compaction.
-   *
-   * Default: {@link AccessHint#NORMAL}
-   *
-   * @param accessHint The access hint
-   *
-   * @return the reference to the current options.
-   */
-  @Deprecated T setAccessHintOnCompactionStart(final AccessHint accessHint);
-
-  /**
-   * Specify the file access pattern once a compaction is started.
-   * It will be applied to all input files of a compaction.
-   *
-   * Default: {@link AccessHint#NORMAL}
-   *
-   * @return The access hint
-   */
-  @Deprecated AccessHint accessHintOnCompactionStart();
-
-  /**
-   * This is a maximum buffer size that is used by WinMmapReadableFile in
-   * unbuffered disk I/O mode. We need to maintain an aligned buffer for
-   * reads. We allow the buffer to grow until the specified value and then
-   * for bigger requests allocate one shot buffers. In unbuffered mode we
-   * always bypass read-ahead buffer at ReadaheadRandomAccessFile
-   * When read-ahead is required we then make use of
-   * {@link MutableDBOptionsInterface#compactionReadaheadSize()} value and
-   * always try to read ahead.
-   * With read-ahead we always pre-allocate buffer to the size instead of
-   * growing it up to a limit.
-   *
-   * This option is currently honored only on Windows
-   *
-   * Default: 1 Mb
-   *
-   * Special value: 0 - means do not maintain per instance buffer. Allocate
-   *                per request buffer and avoid locking.
-   *
-   * @param randomAccessMaxBufferSize the maximum size of the random access
-   *     buffer
-   *
-   * @return the reference to the current options.
-   */
-  T setRandomAccessMaxBufferSize(long randomAccessMaxBufferSize);
-
-  /**
-   * This is a maximum buffer size that is used by WinMmapReadableFile in
-   * unbuffered disk I/O mode. We need to maintain an aligned buffer for
-   * reads. We allow the buffer to grow until the specified value and then
-   * for bigger requests allocate one shot buffers. In unbuffered mode we
-   * always bypass read-ahead buffer at ReadaheadRandomAccessFile
-   * When read-ahead is required we then make use of
-   * {@link MutableDBOptionsInterface#compactionReadaheadSize()} value and
-   * always try to read ahead. With read-ahead we always pre-allocate buffer
-   * to the size instead of growing it up to a limit.
-   *
-   * This option is currently honored only on Windows
-   *
-   * Default: 1 Mb
-   *
-   * Special value: 0 - means do not maintain per instance buffer. Allocate
-   *                per request buffer and avoid locking.
-   *
-   * @return the maximum size of the random access buffer
-   */
-  long randomAccessMaxBufferSize();
-
   /**
    * Use adaptive mutex, which spins in the user space before resorting
    * to kernel. This could reduce context switch when the mutex is not
@@ -1766,4 +1696,47 @@ public interface DBOptionsInterface<T extends DBOptionsInterface<T>> {
    * @return the instance of the current object.
    */
   long bgerrorResumeRetryInterval();
+
+  /**
+   * Implementing off-peak duration awareness in RocksDB. In this context,
+   * "off-peak time" signifies periods characterized by significantly less read
+   * and write activity compared to other times. By leveraging this knowledge,
+   * we can prevent low-priority tasks, such as TTL-based compactions, from
+   * competing with read and write operations during peak hours. Essentially, we
+   * preprocess these tasks during the preceding off-peak period, just before
+   * the next peak cycle begins. For example, if the TTL is configured for 25
+   * days, we may compact the files during the off-peak hours of the 24th day.
+   *
+   * Time of the day in UTC, start_time-end_time inclusive.
+   * Format - HH:mm-HH:mm (00:00-23:59)
+   * If the start time exceeds the end time, it will be considered that the time period
+   * spans to the next day (e.g., 23:30-04:00). To make an entire day off-peak,
+   * use "0:00-23:59". To make an entire day have no offpeak period, leave
+   * this field blank. Default: Empty string (no offpeak).
+   *
+   * @param offpeakTimeUTC String value from which to parse offpeak time range
+   */
+  T setDailyOffpeakTimeUTC(final String offpeakTimeUTC);
+
+  /**
+   *
+   * Implementing off-peak duration awareness in RocksDB. In this context,
+   * "off-peak time" signifies periods characterized by significantly less read
+   * and write activity compared to other times. By leveraging this knowledge,
+   * we can prevent low-priority tasks, such as TTL-based compactions, from
+   * competing with read and write operations during peak hours. Essentially, we
+   * preprocess these tasks during the preceding off-peak period, just before
+   * the next peak cycle begins. For example, if the TTL is configured for 25
+   * days, we may compact the files during the off-peak hours of the 24th day.
+   *
+   * Time of the day in UTC, start_time-end_time inclusive.
+   * Format - HH:mm-HH:mm (00:00-23:59)
+   * If the start time exceeds the end time, it will be considered that the time period
+   * spans to the next day (e.g., 23:30-04:00). To make an entire day off-peak,
+   * use "0:00-23:59". To make an entire day have no offpeak period, leave
+   * this field blank. Default: Empty string (no offpeak).
+   *
+   * @return String value of current offpeak time range, "" if none is set.
+   */
+  String dailyOffpeakTimeUTC();
 }
