@@ -75,20 +75,49 @@ public class EventListenerTest {
 
   void deleteTableFile(final AbstractEventListener el, final AtomicBoolean wasCbCalled)
       throws RocksDBException {
-    try (final Options opt =
-             new Options().setCreateIfMissing(true).setListeners(Collections.singletonList(el));
+    final int KEY_SIZE = 20;
+    final int VALUE_SIZE = 1000;
+    final int FILE_SIZE = 64000;
+    final int NUM_FILES = 2;
+
+    final int KEY_INTERVAL = 10000;
+    /*
+     * Intention of these options is to end up reliably with NUM_FILES files.
+     * we will be deleting using deleteFilesInRange.
+     * It is writing roughly number of keys that will fit in NUM_FILES files (target size).
+     * It is writing interleaved so that files from memory on L0 will overlap.
+     * Then compaction cleans everything, and we should end up with NUM_FILES files.
+     */
+    try (final Options opt = new Options()
+                                 .setCreateIfMissing(true)
+                                 .setListeners(Collections.singletonList(el))
+                                 .setCompressionType(CompressionType.NO_COMPRESSION)
+                                 .setTargetFileSizeBase(FILE_SIZE)
+                                 .setWriteBufferSize(FILE_SIZE / 2)
+                                 .setDisableAutoCompactions(true)
+                                 .setLevelCompactionDynamicLevelBytes(false);
          final RocksDB db = RocksDB.open(opt, dbFolder.getRoot().getAbsolutePath())) {
-      assertThat(db).isNotNull();
-      final byte[] value = new byte[24];
-      rand.nextBytes(value);
-      db.put("testKey".getBytes(), value);
-      final RocksDB.LiveFiles liveFiles = db.getLiveFiles();
-      assertThat(liveFiles).isNotNull();
-      assertThat(liveFiles.files).isNotNull();
-      assertThat(liveFiles.files.isEmpty()).isFalse();
-      db.deleteFile(liveFiles.files.get(0));
-      assertThat(wasCbCalled.get()).isTrue();
+      final int records = FILE_SIZE / (KEY_SIZE + VALUE_SIZE);
+
+      // fill database with key/value pairs
+      final byte[] value = new byte[VALUE_SIZE];
+      int key_init = 0;
+      for (int o = 0; o < NUM_FILES; ++o) {
+        int int_key = key_init++;
+        for (int i = 0; i < records; ++i) {
+          int_key += KEY_INTERVAL;
+          rand.nextBytes(value);
+
+          db.put(String.format("%020d", int_key).getBytes(), value);
+        }
+      }
+      try (final FlushOptions flushOptions = new FlushOptions().setWaitForFlush(true)) {
+        db.flush(flushOptions);
+      }
+      db.compactRange();
+      db.deleteFilesInRanges(null, Arrays.asList(null, null), false /* includeEnd */);
     }
+    assertThat(wasCbCalled.get()).isTrue();
   }
 
   @Test
