@@ -13,12 +13,12 @@ import static org.assertj.core.api.Assertions.assertThat;
 public class PlainTableConfigTest {
 
   @ClassRule
-  public static final RocksMemoryResource rocksMemoryResource =
-      new RocksMemoryResource();
+  public static final RocksNativeLibraryResource ROCKS_NATIVE_LIBRARY_RESOURCE =
+      new RocksNativeLibraryResource();
 
   @Test
   public void keySize() {
-    PlainTableConfig plainTableConfig = new PlainTableConfig();
+    final PlainTableConfig plainTableConfig = new PlainTableConfig();
     plainTableConfig.setKeySize(5);
     assertThat(plainTableConfig.keySize()).
         isEqualTo(5);
@@ -26,7 +26,7 @@ public class PlainTableConfigTest {
 
   @Test
   public void bloomBitsPerKey() {
-    PlainTableConfig plainTableConfig = new PlainTableConfig();
+    final PlainTableConfig plainTableConfig = new PlainTableConfig();
     plainTableConfig.setBloomBitsPerKey(11);
     assertThat(plainTableConfig.bloomBitsPerKey()).
         isEqualTo(11);
@@ -34,7 +34,7 @@ public class PlainTableConfigTest {
 
   @Test
   public void hashTableRatio() {
-    PlainTableConfig plainTableConfig = new PlainTableConfig();
+    final PlainTableConfig plainTableConfig = new PlainTableConfig();
     plainTableConfig.setHashTableRatio(0.95);
     assertThat(plainTableConfig.hashTableRatio()).
         isEqualTo(0.95);
@@ -42,7 +42,7 @@ public class PlainTableConfigTest {
 
   @Test
   public void indexSparseness() {
-    PlainTableConfig plainTableConfig = new PlainTableConfig();
+    final PlainTableConfig plainTableConfig = new PlainTableConfig();
     plainTableConfig.setIndexSparseness(18);
     assertThat(plainTableConfig.indexSparseness()).
         isEqualTo(18);
@@ -50,7 +50,7 @@ public class PlainTableConfigTest {
 
   @Test
   public void hugePageTlbSize() {
-    PlainTableConfig plainTableConfig = new PlainTableConfig();
+    final PlainTableConfig plainTableConfig = new PlainTableConfig();
     plainTableConfig.setHugePageTlbSize(1);
     assertThat(plainTableConfig.hugePageTlbSize()).
         isEqualTo(1);
@@ -58,7 +58,7 @@ public class PlainTableConfigTest {
 
   @Test
   public void encodingType() {
-    PlainTableConfig plainTableConfig = new PlainTableConfig();
+    final PlainTableConfig plainTableConfig = new PlainTableConfig();
     plainTableConfig.setEncodingType(EncodingType.kPrefix);
     assertThat(plainTableConfig.encodingType()).isEqualTo(
         EncodingType.kPrefix);
@@ -66,13 +66,13 @@ public class PlainTableConfigTest {
 
   @Test
   public void fullScanMode() {
-    PlainTableConfig plainTableConfig = new PlainTableConfig();
+    final PlainTableConfig plainTableConfig = new PlainTableConfig();
     plainTableConfig.setFullScanMode(true);
     assertThat(plainTableConfig.fullScanMode()).isTrue();  }
 
   @Test
   public void storeIndexInFile() {
-    PlainTableConfig plainTableConfig = new PlainTableConfig();
+    final PlainTableConfig plainTableConfig = new PlainTableConfig();
     plainTableConfig.setStoreIndexInFile(true);
     assertThat(plainTableConfig.storeIndexInFile()).
         isTrue();
