@@ -72,7 +72,8 @@ constexpr bool kMustFreeHeapAllocations = false;
 // Compile-time CPU feature testing compatibility
 //
 // A way to be extra sure these defines have been included.
-#define ASSERT_FEATURE_COMPAT_HEADER() /* empty */
+#define ASSERT_FEATURE_COMPAT_HEADER() \
+  static_assert(true, "Semicolon required") /* empty */
 
 // MSVC doesn't support the same defines that gcc and clang provide
 // but does some like __AVX__. Here we can infer some features from others.
