@@ -319,10 +319,12 @@ class BlockFetcherTest : public testing::Test {
     PersistentCacheOptions persistent_cache_options;
     Footer footer;
     ReadFooter(file, &footer);
+    auto mgr = GetBuiltinCompressionManager(
+        GetCompressFormatForVersion(footer.format_version()));
     std::unique_ptr<BlockFetcher> fetcher(new BlockFetcher(
         file, nullptr /* prefetch_buffer */, footer, roptions, block, contents,
         ioptions, do_uncompress, compressed, block_type,
-        UncompressionDict::GetEmptyDict(), persistent_cache_options,
+        mgr->GetDecompressor().get(), persistent_cache_options,
         heap_buf_allocator, compressed_buf_allocator));
 
     ASSERT_OK(fetcher->ReadBlockContents());
@@ -335,7 +337,7 @@ class BlockFetcherTest : public testing::Test {
     if (do_uncompress) {
       *compression_type = kNoCompression;
     } else {
-      *compression_type = fetcher->get_compression_type();
+      *compression_type = fetcher->compression_type();
     }
   }
 
@@ -530,7 +532,6 @@ TEST_F(BlockFetcherTest, FetchAndUncompressCompressedDataBlock) {
                      expected_stats_by_mode);
 }
 
-
 }  // namespace
 }  // namespace ROCKSDB_NAMESPACE
 
