@@ -69,7 +69,7 @@ class PointLockManagerTest : public testing::Test {
   PessimisticTransaction* NewTxn(
       TransactionOptions txn_opt = TransactionOptions()) {
     Transaction* txn = db_->BeginTransaction(WriteOptions(), txn_opt);
-    return reinterpret_cast<PessimisticTransaction*>(txn);
+    return static_cast<PessimisticTransaction*>(txn);
   }
 
  protected:
