@@ -42,7 +42,8 @@ BaseDeltaIterator::BaseDeltaIterator(ColumnFamilyHandle* column_family,
 BaseDeltaIterator::~BaseDeltaIterator() = default;
 
 bool BaseDeltaIterator::Valid() const {
-  return status_.ok() ? (current_at_base_ ? BaseValid() : DeltaValid()) : false;
+  return status().ok() ? (current_at_base_ ? BaseValid() : DeltaValid())
+                       : false;
 }
 
 void BaseDeltaIterator::SeekToFirst() {
@@ -89,14 +90,22 @@ void BaseDeltaIterator::Next() {
     equal_keys_ = false;
     if (!BaseValid()) {
       assert(DeltaValid());
+      // During reverse scan, base iter was exhausted. This means the
+      // base iter has no smaller key than the current key at delta iter.
+      // So seeking to the first key of base iter will point to base iter's
+      // first key that is at least the current key.
       base_iterator_->SeekToFirst();
     } else if (!DeltaValid()) {
+      // Similar to the base iter case above, position delta iter to the first
+      // entry that is at least the current key.
       delta_iterator_->SeekToFirst();
     } else if (current_at_base_) {
-      // Change delta from larger than base to smaller
+      // current_at_base_ means delta iter is at its first key that is less
+      // than current key. So advance delta one step moves it to the first
+      // key that is at least the current key.
       AdvanceDelta();
     } else {
-      // Change base from larger than delta to smaller
+      // Similar to the case above.
       AdvanceBase();
     }
     if (DeltaValid() && BaseValid()) {
@@ -289,6 +298,9 @@ void BaseDeltaIterator::SetValueAndColumnsFromDelta() {
   WriteEntry delta_entry = delta_iterator_->Entry();
 
   if (merge_context_.GetNumOperands() == 0) {
+    // delete case is handled at callsites, see UpdateCurrent().
+    assert(delta_entry.type == kPutRecord ||
+           delta_entry.type == kPutEntityRecord);
     if (delta_entry.type == kPutRecord) {
       value_ = delta_entry.value;
       columns_.emplace_back(kDefaultWideColumnName, value_);
@@ -411,13 +423,14 @@ void BaseDeltaIterator::UpdateCurrent() {
         return;
       }
 
-      // Base has finished.
       if (!DeltaValid()) {
-        // Finished
+        // Finished, neither base nor delta is valid.
         return;
       }
+      // Base is done, delta has value.
       if (delta_result == WBWIIteratorImpl::kDeleted &&
           merge_context_.GetNumOperands() == 0) {
+        // This key is deleted and no Merge is done after Delete.
         AdvanceDelta();
       } else {
         current_at_base_ = false;
@@ -425,13 +438,14 @@ void BaseDeltaIterator::UpdateCurrent() {
         return;
       }
     } else if (!DeltaValid()) {
-      // Delta has finished.
+      // Delta is done, base has value.
       current_at_base_ = true;
       if (!allow_unprepared_value_) {
         SetValueAndColumnsFromBase();
       }
       return;
     } else {
+      // Both are valid.
       int compare =
           (forward_ ? 1 : -1) * comparator_->CompareWithoutTimestamp(
                                     delta_entry.key, /*a_has_ts=*/false,
@@ -442,6 +456,7 @@ void BaseDeltaIterator::UpdateCurrent() {
         }
         if (delta_result != WBWIIteratorImpl::kDeleted ||
             merge_context_.GetNumOperands() > 0) {
+          // delta is visible
           current_at_base_ = false;
           SetValueAndColumnsFromDelta();
           return;
@@ -452,6 +467,7 @@ void BaseDeltaIterator::UpdateCurrent() {
           AdvanceBase();
         }
       } else {
+        // base smaller than delta
         current_at_base_ = true;
         if (!allow_unprepared_value_) {
           SetValueAndColumnsFromBase();
@@ -459,12 +475,15 @@ void BaseDeltaIterator::UpdateCurrent() {
         return;
       }
     }
+    // delta <= base and is not visible (most recent update is delete)
+    // move on to the next key
   }
 
   AssertInvariants();
 #endif  // __clang_analyzer__
 }
 
+// Move forward/backward until the iterator is at a different key.
 void WBWIIteratorImpl::AdvanceKey(bool forward) {
   if (Valid()) {
     Slice key = Entry().key;
@@ -515,18 +534,17 @@ WBWIIteratorImpl::Result WBWIIteratorImpl::FindLatestUpdate(
              0) {
     return result;
   } else {
-    // We want to iterate in the reverse order that the writes were added to the
-    // batch.  Since we don't have a reverse iterator, we must seek past the
-    // end. We do this by seeking to the next key, and then back one step
-    NextKey();
+    // Need to move to the first entry of the current key
+    // We may not always be at the first entry, e.g., after SeekToLast() or a
+    // SeekForPrev().
+    AdvanceKey(false);
     if (Valid()) {
-      Prev();
+      Next();
     } else {
-      SeekToLast();
+      SeekToFirst();
     }
-
-    // We are at the end of the iterator for this key.  Search backwards for the
-    // last Put or Delete, accumulating merges along the way.
+    // We are at the first and most recent entry for this key, search forward
+    // until a non-Merge entry, accumulating merges along the way.
     while (Valid()) {
       const WriteEntry entry = Entry();
       if (comparator_->CompareKey(column_family_id_, entry.key, key) != 0) {
@@ -551,18 +569,21 @@ WBWIIteratorImpl::Result WBWIIteratorImpl::FindLatestUpdate(
         case kPutEntityRecord:
           return WBWIIteratorImpl::kFound;
         default:
+          assert(false);
           return WBWIIteratorImpl::kError;
       }  // end switch statement
-      Prev();
+      Next();
     }  // End while Valid()
     // At this point, we have been through the whole list and found no Puts or
-    // Deletes. The iterator points to the previous key.  Move the iterator back
-    // onto this one.
+    // Deletes. The iterator points to the next key.  Move the iterator back
+    // to the last entry of this key (see comment for FindLatestUpdate()).
     if (Valid()) {
-      Next();
+      Prev();
     } else {
-      SeekToFirst();
+      SeekToLast();
     }
+    assert(Valid());
+    assert(comparator_->CompareKey(column_family_id_, Entry().key, key) == 0);
   }
   return result;
 }
@@ -650,7 +671,8 @@ Status ReadableWriteBatch::GetEntryFromDataOffset(size_t data_offset,
 // 2. Inside the same CF, we first decode the entry to find the key of the entry
 //    and the entry with larger key will be larger;
 // 3. If two entries are of the same CF and key, the one with larger offset
-//    will be larger.
+//    will be smaller. This follows the internal key order where keys
+//    with larger sequence number (larger offset here) will be ordered first.
 // Some times either `entry1` or `entry2` is dummy entry, which is actually
 // a search key. In this case, in step 2, we don't go ahead and decode the
 // entry but use the value in WriteBatchIndexEntry::search_key.
@@ -691,9 +713,9 @@ int WriteBatchEntryComparator::operator()(
   int cmp = CompareKey(entry1->column_family, key1, key2);
   if (cmp != 0) {
     return cmp;
-  } else if (entry1->offset > entry2->offset) {
-    return 1;
   } else if (entry1->offset < entry2->offset) {
+    return 1;
+  } else if (entry1->offset > entry2->offset) {
     return -1;
   }
   return 0;
